@@ -8,17 +8,47 @@ import glob
 import re
 import datetime
 from calendar import timegm
+from collections import defaultdict
+from itertools import chain
 import _dbdreader
 import logging
 
+# make sure we interpret timestamps in the english language
+locale.setlocale(locale.LC_ALL, 'en_US.UTF-8')
+
 logger = logging.getLogger(os.path.basename(__file__))
 logging.basicConfig(level=logging.INFO)
 
-# make sure we interpret timestamps in the english language
-try:
-    locale.setlocale(locale.LC_ALL, 'en_US')
-except:
-    pass # building on read-the-docs fails because of this.
+LATLON_PARAMS = ["m_lat",
+                 "m_lon",
+                 "c_wpt_lat",
+                 "c_wpt_lon",
+                 "x_last_wpt_lat",
+                 "x_last_wpt_lon",
+                 "m_gps_lat",
+                 "m_gps_lon",
+                 "u_lat_goto_l99",
+                 "u_lon_goto_l99",
+                 "m_last_gps_lat_1",
+                 "m_last_gps_lon_1",
+                 "m_last_gps_lat_2",
+                 "m_last_gps_lon_2",
+                 "m_last_gps_lat_3",
+                 "m_last_gps_lon_3",
+                 "m_last_gps_lat_4",
+                 "m_last_gps_lon_4",
+                 "m_gps_ignored_lat",
+                 "m_gps_ignored_lon",
+                 "m_gps_invalid_lat",
+                 "m_gps_invalid_lon",
+                 "m_gps_toofar_lat",
+                 "m_gps_toofar_lon",
+                 "xs_lat",
+                 "xs_lon",
+                 "s_ini_lat",
+                 "s_ini_lon"]
+
+
 
 def strptimeToEpoch(datestr, fmt):
     ''' Converts datestr into seconds
@@ -367,6 +397,9 @@ class DBDPatternSelect(object):
         ValueError if nor pattern or filenames is given.
         '''
         fns = self.get_filenames(pattern, filenames)
+        if not fns:
+            raise DbdError(DBD_ERROR_NO_FILES_FOUND,
+                           f"No files matched search pattern {pattern}.")
         if t_start is None:
             t_start = numpy.min(list(self.cache.keys()))
         if t_end is None:
@@ -411,7 +444,6 @@ class DBDPatternSelect(object):
             if fn not in cached_filenames:
                 dbd=DBD(fn, cacheDir)
                 t_open=dbd.get_fileopen_time()
-                dbd.close()
                 self.cache[t_open]=fn
                 
     def __select(self,all_fns,t0,t1):
@@ -508,7 +540,8 @@ class DBD(object):
     cachedDir: str or None, optional
         path to CAC file cache directory. If None, the default path is used.
     '''
-    def __init__(self,filename,cacheDir=None):
+
+    def __init__(self,filename,cacheDir=None, raise_exception_if_cac_not_found=True):
 
         self.filename=filename
         self.fp=open(filename,'br')
@@ -525,6 +558,9 @@ class DBD(object):
         self.parameterNames=[i[1] for i in parameterInfo]
         self.parameterUnits=dict((i[1],i[2]) for i in parameterInfo)
         self.timeVariable=self.__set_timeVariable()
+        self.close()
+        if not self.cacheFound and raise_exception_if_cac_not_found:
+            raise DbdError(DBD_ERROR_CACHE_NOT_FOUND, f" ({self.cacheID}.cac)")
 
     # def getpy(self, parameter):
     #     ti = self.parameterNames.index(self.timeVariable)
@@ -773,11 +809,16 @@ class DBD(object):
                 idx = numpy.where(numpy.isclose(values[i],1e9))[0]
                 values[i][idx] = numpy.nan
             if self.__is_latlon_parameter(p):
+                if discardBadLatLon and not return_nans: #discards and return nans is not compatible.
+                    # p is either a latitude or longitude parameter. Check now which one it is.
+                    if "lat" in p: 
+                        value_limit = 9000 # nmea style
+                    else:
+                        value_limit = 18000 # nmea style
+                    condition = numpy.logical_and(values[i]>=-value_limit, values[i]<=value_limit)
+                    timestamps[i], values[i] = numpy.compress(condition, (timestamps[i], values[i]), axis=1)
                 if decimalLatLon:
                     values[i] = toDec(values[i])
-                if discardBadLatLon and not return_nans: #discards and return nans is not compatible.
-                    condition = values[i]<696960
-                    timestamps[i], values[i] = numpy.compress(condition, (timestamps[i], values[i]), axis=1)
         return timestamps, values
 
 
@@ -817,10 +858,7 @@ class DBD(object):
         return validParameters
 
     def __is_latlon_parameter(self,x):
-        if 'lat' in x or 'lon' in x:
-            return True
-        else:
-            return False
+        return x in LATLON_PARAMS
 
     def __read_header(self, cacheDir):
         if not os.path.exists(cacheDir):
@@ -1509,22 +1547,38 @@ class MultiDBD(object):
     def __update_dbd_inventory(self, cacheDir):
         self.dbds={'eng':[],'sci':[]}
         filenames=list(self.filenames)
+        missing_cacheIDs = []
         for fn in self.filenames:
-            dbd=DBD(fn, cacheDir)
+            dbd=DBD(fn, cacheDir, raise_exception_if_cac_not_found=False)
+            if not dbd.cacheFound:
+                missing_cacheIDs.append(dbd.cacheID) # add missing cacheId for and check later if we need it
             mission_name=dbd.get_mission_name()
-            dbd.close()
             if mission_name in self.banned_missions:
                 filenames.remove(fn)
                 continue
             if self.missions and mission_name not in self.missions:
                 filenames.remove(fn)
                 continue
+            # so we decided to keep the file. Continue only when the cache file was found.
             if mission_name not in self.mission_list:
                 self.mission_list.append(mission_name)
             if self.isScienceDataFile(fn):
                 self.dbds['sci'].append(dbd)
             else:
                 self.dbds['eng'].append(dbd)
+        # Now check if any of the remaining files depends on a missing cache file.
+        problems = defaultdict(list)
+        for dbd in chain(*self.dbds.values()):
+            if dbd.cacheID in missing_cacheIDs:
+                problems[dbd.cacheID].append(dbd.filename)
+        if problems:
+            mesg = "\n" # craft some useful error message
+            for k, v in problems.items():
+                mesg+=f"{k} : {v[0]}"
+                if len(v)>1:
+                    mesg += f" + {len(v)-1} more files."
+                mesg+="\n"
+            raise DbdError(DBD_ERROR_CACHE_NOT_FOUND, mesg)
         if len(self.dbds['sci'])+len(self.dbds['eng'])==0:
             raise DbdError(DBD_ERROR_ALL_FILES_BANNED, " (Read %d files.)"%(len(self.filenames)))
         self.filenames=filenames
